'use strict';

exports.__esModule = true;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ = require('../');

var _2 = _interopRequireDefault(_);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var compile = function compile(pathIn, pathOut, opts) {
  var ignoreChecks = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

  var file = _fs2['default'].lstatSync(pathIn);
  if (file.isFile()) {
    var compilable = true;
    if (!ignoreChecks) {
      if (compilable && opts.extensions) {
        compilable = opts.extensions.reduce(function (res, ext) {
          return res || pathIn.lastIndexOf(ext) === pathIn.length - ext.length;
        }, false);
      }
      if (compilable && opts.ignore) {
        compilable = !new RegExp(opts.ignore).test(pathIn);
      }
    }
    if (compilable) {
      var code = _2['default'].parse(_fs2['default'].readFileSync(pathIn, 'utf8'));
      if (pathOut) {
        _fs2['default'].writeFileSync(pathOut, code);
        // eslint-disable-next-line
        console.log(pathIn + ' -> ' + pathOut);
      } else {
        // eslint-disable-next-line
        console.log(code);
      }
    } else if (opts.copyFiles) {
      return (0, _utils.copy)(pathIn, pathOut);
    }
  } else if (file.isDirectory()) {
    var compiles = _fs2['default'].readdirSync(pathIn).map(function (subPath) {
      var pathInSub = _path2['default'].join(pathIn, subPath);
      var pathOutSub = _path2['default'].join(pathOut, subPath);

      if (_fs2['default'].lstatSync(pathInSub).isDirectory()) {
        if (!_fs2['default'].existsSync(pathOutSub)) {
          _fs2['default'].mkdirSync(pathOutSub);
        }
      }

      return compile(pathInSub, pathOutSub, opts);
    });
    return Promise.all(compiles);
  }
  return Promise.resolve();
};

exports['default'] = compile;