'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports['default'] = buildConfig;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var defaultConfig = {
  output: undefined,
  extensions: ['.cpx', '.CPX', '.C', '.cc', '.cpp', '.CPP', '.c++', '.cp', '.cxx'],
  ignore: undefined,
  gccxrc: true,
  copyFiles: true,
  watch: false
};

var mergeConfig = function mergeConfig(config1, config2) {
  var config = {};
  Object.keys(defaultConfig).forEach(function (key) {
    if (config2[key] !== undefined) {
      config[key] = config2[key];
    }
  });
  return Object.assign({}, config1, config);
};

function buildConfig(input) {
  var program = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (typeof input !== 'string') {
    throw new TypeError('input has to be a string, got: ' + (typeof input === 'undefined' ? 'undefined' : _typeof(input)));
  } else if (input === '') {
    throw new Error('input cannot be empty');
  } else if (!_fs2['default'].existsSync(input)) {
    throw new Error('input is not a file or a directory');
  }

  var config = _extends({
    input: input
  }, defaultConfig);

  var gccxrcPath = _path2['default'].join(process.cwd(), '.gccxrc');
  if ((program.gccxrc || program.gccxrc === undefined) && _fs2['default'].existsSync(gccxrcPath)) {
    var gccxrcConfigs = JSON.parse(_fs2['default'].readFileSync(gccxrcPath, 'utf8'));
    config = mergeConfig(config, gccxrcConfigs);
  }

  config = mergeConfig(config, program);

  config.extensions.forEach(function (ext) {
    if (!/^\.[a-zA-Z_][a-zA-Z0-9_]*/.test(ext)) {
      throw new Error('extension "' + ext + '" does not match the extension format');
    }
  });

  var inputIsDirectory = _fs2['default'].lstatSync(input).isDirectory();
  if (inputIsDirectory && !config.output) {
    throw new Error('invalid config: cannot use input directory without output option');
  } else if (inputIsDirectory && /(\.\w+)+/.test(config.output)) {
    throw new Error('invalid config: output is a file and input is a directory');
  }

  return config;
}